# Send it to a friend to test

Send the Dec App installation link corresponding to the `CYFS App Install Link` in the previous chapter `Implementing the Front End` to your friends, and then follow the steps in the previous chapter `Go to the CYFS browser to install the Dec App and view` , let your friends also successfully install our message board Dec App!

## try posting a message

You and your friends can try to post a message, and each of you will see your latest message on your message board.

# Want to see friend's message board

I'm sure you're not content with just looking at your own message boards, that's kind of boring, isn't it? You must be curious about what your friends have written on your message boards, and want to see your friends' message boards.
To do this, you have to be able to set the friends' peopleId in one place. To make this as simple as possible, we will set the peopleid value in the search in the page's URL to the friend's peopleId.
In general, to achieve this function, only 2 steps are required:

1. Open the necessary permissions in the Service.
2. Added URL detection function and modified request method to the front-end page.

## Open the necessary permissions in the Service

### First time facing permission problem

In CYFS, each request contains two pieces of key information: **source and target**. The protocol stack determines whether it matches by obtaining the acl configuration of the target according to the source.
For a request, it contains key four-tuple information: (source-device-id, source-dec-id) -> (target-device-id, target-dec-id), the permission system judges the request based on the four-tuple Whether the permission is satisfied, so as to decide whether to release.

#### Classification of permission system

In CYFS, the permission control of Dec App is divided into 2 categories:

1. Apply for your own path permission.
   If you open permissions to yourself, you can set it in the acl.toml configuration file, or you can register it dynamically in the code.
   Dynamic registration is to configure the corresponding permissions for the specified req_path during the dec_ervice operation. To configure permissions for req_path is to set access-strings corresponding to different req_paths.
2. Apply for other apps to open permissions for yourself.
   If you want to request other apps to open permissions to yourself, you must write it in the acl.toml configuration file, and you cannot register it dynamically in the code.
   After configuring the acl.toml file, the file will be read by app-manager when the app is installed, and the corresponding permissions will be registered with the system according to the configured values.

##### Apply for your own path permission

The access-string is designed with reference to the permissions of the file system under LINUX, and uses u32 to represent the specific permissions of a specific group.

#### Permission classification

Currently, permissions are divided into three types: Read, Write, and Call.
When setting permissions, you can set a single permission, that is, AccessPermission, or you can set it by combining multiple permissions to reduce the amount of code, that is, AccessPermissions.
Looking at the declaration file of `cyfs-sdk`, you can see that the source code corresponding to AccessPermission and AccessPermissions is as follows:

```typescript
export class AccessPermission {
	static Call: AccessPermission;
	static Write: AccessPermission;
	static Read: AccessPermission;
}
export class AccessPermissions {
	static None: AccessPermissions;
	static CallOnly: AccessPermissions;
	static WriteOnly: AccessPermissions;
	static WirteAndCall: AccessPermissions;
	static ReadOnly: AccessPermissions;
	static ReadAndCall: AccessPermissions;
	static ReadAndWrite: AccessPermissions;
	static Full: AccessPermissions;
}
```

#### Permission grouping

Currently, it is divided into six groups according to zone, device and dec:

1. CurrentZone: Devices in the current zone (all devices bound to the same people)
2. FriendZone: friend device, this is all devices under People in the friend list
3. OthersZone: All unknown devices except current zone and friend zone
4. CurrentDevice: current device
5. OwnerDec: all dec of people in the current Zone
6. OthersDec: dec for all other locations outside the Zone

Each group can set 3 independent permissions in the previous section `Permission Classification`:

```typescript
export class AccessGroup {
	static CurrentDevice: AccessGroup;
	static CurrentZone: AccessGroup;
	static FriendZone: AccessGroup;
	static OthersZone: AccessGroup;
	static OwnerDec: AccessGroup;
	static OthersDec: AccessGroup;
}
```

#### How to operate

We can use AccessString to generate the corresponding permissions. For example, if you want to set the permission to be read by all decs in the same zone, you should refer to the following code to set:

```typescript
import * as cyfs from "cyfs-sdk";

const access = new cyfs.AccessString(0);
access.set_group_permissions(
	cyfs.AccessGroup.CurrentZone,
	cyfs.AccessPermissions.ReadOnly
);
access.set_group_permissions(
	cyfs.AccessGroup.CurrentDevice,
	cyfs.AccessPermissions.ReadOnly
);
access.set_group_permissions(
	cyfs.AccessGroup.OwnerDec,
	cyfs.AccessPermissions.ReadOnly
);
access.set_group_permissions(
	cyfs.AccessGroup.OtherDec,
	cyfs.AccessPermissions.ReadOnly
);
```

#### Apply for other apps to open permissions for yourself

##### Why apply for permission

If it is within the same Zone, the root-state read/write, add-handler and post_object between the same Dec do not need to configure any additional permissions.

If there is a need for cross-zone or cross-dec, explicit configuration permissions are required:

- Operation on root-state: configure r or w permissions according to read/write requirements

- add-handler: If you want to cross Zone or add-handler of Dec, you need to configure the corresponding write permission for the virtual path of add-handler

- post_object: If you want to go to post_object across Zone or across Dec, the receiver must configure execution permission for this virtual path, that is, x permission

##### Configuration method

Currently, there are two ways to represent an AccessString in the acl.toml configuration file:

1. For a complete string, an 18-bit string is used to represent a complete permission. In the group, "rwx-" of linux is used to represent the permission of each bit. Groups can be separated by spaces or underscores

   > Example: Give CurrentDevice, OwnerDec full permission of CurrentZone, read and write permission to OwnerDec of FriendZone, and read permission of OthersDec of OthersZone:

   > The string representing the above permission is "rwxrwxrw-r--rwxr--", which is the same as "rwx rwx rw- r-- rwx r--", and "rwx_rwx_rw-\_r--\_rwx_r--" equivalent

2. Based on the default permissions, the permissions are individually marked for certain groups: represented as an array, the array is {group, access}, group is the enumeration name of AccessGroup, and access is a three-digit "rwx-" string

   > Default AccessString permission: "rwxrwxrwx---rwx"

   > Take the above permissions as an example, expressed as `[{group = "FriendZone", access = "rw-"}, {group = "OthersZone", access = "r--"}, {group = "OthersDec" , access = "r--"}]`

##### A complete acl.toml instance

---

The top-level key is the dec id that needs to apply for permission, and `self` is a special key that represents your own dec id.

---

```toml

[self]

[self.access] // Configure the permissions of your own three paths

// /test3 configure permissions using individual notation

"/test3" = [{group = "OthersDec", access = "-wx"}, {group = "CurrentDevice", access = "---"}]

// The next two paths use full string notation to configure permissions

"/test2" = "rwxrwxrwx---rwx---"

"/test1" = "rwxrwxrwx---rwx--x"



[self.specified] // Open permissions to other dec

"/test3" = {access = "--x", dec_id = "9tGpLNnDpa8deXEk2NaWGccEu4yFQ2DrTZJPLYLT7gj4"} // Open /test3 call permission to a specific dec

"/test2" = {access = "--x", zone_category = "current-zone", dec_id = "9tGpLNnDpa8deXEk2NaWGccEu4yFQ2DrTZJPLYLT7gj4"} // Open the call permission of /test2 to a specific dec, and can only be called in the current zone

// Open the call permission of /test1 to all decs in a specific zone

"/test1" = {access = "--x", zone = "5aSixgLwnWbmcDKwBtTBd7p9U4bmqwNU2C6h6SCvfMMh"}



// Apply for DECID_A permission for yourself

[DECID_A.specified]

// The SpecifiedGroup configuration below does not allow dec_id to be filled in. The dec_id here is limited to itself. Filling in the dec_id field will invalidate the current configuration

"/test3" = {access = "--x"} // Apply for a specific dec /test3 call permission for yourself

"/test2" = {access = "--x", zone_category = "current-zone"} // Apply for a specific dec /test2 call permission for yourself, only allow calls within this zone

"/test1" = {access = "--x", zone = "5aSixgLwnWbmcDKwBtTBd7p9U4bmqwNU2C6h6SCvfMMh"}// Apply for a specific dec /test2 call permission for yourself, only allow a specific zone to initiate calls



[DECID_A.config] //Since the config field is currently empty, this configuration section can be written or not

```

### How to add reasonable permissions

Through the study of the previous chapter `Facing the permission problem for the first time`, we have mastered the basic knowledge of configuring permissions. However, we are not very clear: how should we reasonably configure the permissions of different req_paths?
You must know that it is a very dangerous behavior to open write permissions at will - your dec app data is messed up or even emptied by other malicious dec apps, which will cause serious losses to yourself!
From `permission classification`, we know that permissions are divided into three categories: Read, Write, and Call.
From `permission group`, we know that there are 6 groups in total: CurrentZone, FriendZone, OthersZone, CurrentDevice, OwnerDec, and OthersDec.
Now let's explain how to set these three permissions, so that we can set permissions correctly in development and build a solid firewall for the data in our dec app.

---

Open different levels of permissions for different groups, depending on how much we trust the source group.

---

#### Call permissions

1. Generally speaking, we can completely rely on the Call call initiated by OwnerDec
2. For CurrentZone, CurrentDevice, FriendZone, OthersZone and OthersDec, we can selectively open Call permissions to group some or all trusted sources, or we can consider these sources to be untrustworthy and refuse to open Call permissions.

#### Read permission

1. Generally speaking, we can completely rely on the Read call initiated by OwnerDec
2. For CurrentZone, CurrentDevice, FriendZone, OthersZone and OthersDec, we can selectively open Read permissions to some or all of the trusted sources, or we can consider these sources to be untrustworthy and refuse to open Read permissions.

#### Write permission

1. Generally speaking, we can completely rely on the Write call initiated by OwnerDec
2. For CurrentZone, CurrentDevice, FriendZone, OthersZone and OthersDec, we should carefully open the Write permission to some trusted sources, or consider these sources untrustworthy and refuse to open the Write permission.

### Set permissions for message boards: open permissions for friends

After learning the previous knowledge about `permission`, we will now implement how to open permissions for friends.
Opening permissions for friends requires 2 steps:

1. Open the CallOnly permission of the FriendZone group for querying the message request path `/messages/retrieve`, so that we can directly call the interface for querying the message request path on the friend's Service across Zones.
   Open the src/service/entry/app_startup.ts file, and in the `addRouters` method, uncomment the following code snippet:

   ```typescript
   if (routerObj.reqPath.endsWith("/messages/retrieve")) {
   	access.set_group_permissions(
   		cyfs.AccessGroup.FriendZone,
   		cyfs.AccessPermissions.CallOnly
   	);
   }
   ```

2. Open the read-only permission `ReadOnly` for the OthersZone, CurrentDevice and OwnerDec groups for the `/messages_list` path on RootState.
   Open the src/service/entry/app_startup.ts file, and in the `addRouters` method, uncomment the following code snippet:

   ```typescript
   const access = new cyfs.AccessString(0);
   access.set_group_permissions(
   	cyfs.AccessGroup.OthersZone,
   	cyfs.AccessPermissions.ReadOnly
   );
   access.set_group_permissions(
   	cyfs.AccessGroup.CurrentDevice,
   	cyfs.AccessPermissions.ReadOnly
   );
   access.set_group_permissions(
   	cyfs.AccessGroup.OwnerDec,
   	cyfs.AccessPermissions.ReadOnly
   );
   const r = await stack
   	.root_state_meta_stub()
   	.add_access(cyfs.GlobalStatePathAccessItem.new("/messages_list", access));
   if (r.err) {
   	console.error(`path /messages_list add access error: ${r}`);
   } else {
   	console.log("add sccess /messages_list success.");
   }
   ```

   ##Add URL detection function and modify request method to front-end page

### Modify the src/www/pages/MessageBoard/MessageBoard.tsx page file

1. Modify the `queryMessageRecords` method to remove the code comments on lines 2-3 in the method. Next, modify the parameters that call the `listMessagesByPage` method. In this way, we can specify which friend's message board to view (the URL specifies that the peopleid is set to the friend's poepleId). The modified code is as follows:
   ```typescript
   const target = extractPeopleIdFromSearch();
   const to =
   	target === "self"
   		? undefined
   		: cyfs.ObjectId.from_base_58(target).unwrap();
   const list = await listMessagesByPage(0, to);
   ```
2. Uncomment the code for the `extractPeopleIdFromSearch` method. The function of the `extractPeopleIdFromSearch` method is to detect whether the peopleid in the current URL is specified, and return the friend's peopleid string if the friend's peopleid is specified. Give the code as follows:

   ```typescript
   const extractPeopleIdFromSearch = () => {
   	let target = "self";
   	if (location.search.includes("peopleid=")) {
   		target = location.search.substring(10);
   	}
   	console.log("target: ", target);
   	return target;
   };
   ```

### Modify the src/www/apis/message.ts file

1. Modify the `retrieveMessage` method and add an optional input parameter `target`, whose type is `cyfs.ObjectId`. Then, add the `target` parameter of the `options` object of `postObject`. After adding the `target` parameter to `postObject`, you can initiate requests across zones. code show as below:

   ```typescript
   export async function retrieveMessage(
   	msgKey: string,
   	target?: cyfs.ObjectId
   ) {
   	const stackWraper = checkStack();
   	// Create a new Message object
   	const messageObj = Message.create({
   		key: msgKey,
   		content: "",
   		decId: DEC_ID,
   		owner: stackWraper.checkOwner(),
   	});
   	// make a request
   	const ret = await stackWraper.postObject(messageObj, MessageDecoder, {
   		reqPath: ROUTER_PATHS.RETRIEVE_MESSAGE,
   		decId: DEC_ID,
   		target,
   	});
   	// ...ignore
   }
   ```

2. Modify the `listMessagesByPage` method and add an optional parameter `to`, whose type is `cyfs.ObjectId`. Next, modify the parameter that calls `root_state_access_stub` to `target`. Finally, modify the input parameter `target` that calls the `retrieveMessage` method in the code. After adding the optional parameter `to` to the `listMessagesByPage` method, you can initiate a cross-zone paging request to read the object list under the `/messages_list` path on the friend RootState. code show as below:

```typescript
export async function listMessagesByPage(
	pageIndex: number,
	to?: cyfs.ObjectId
) {
	const stack = checkStack();
	// Get your own OwnerId
	const target = to ? to : stack.checkOwner();
	// Get an instance of cyfs.GlobalStateAccessStub
	const access = stack.check().root_state_access_stub(target);
	// ...ignore
	const msgList = await Promise.all(
		keyList.map(async (item) => {
			const msg = await retrieveMessage(item, target);
			return msg;
		})
	);
	// ...ignore
}
```

## Try to see if you can see your friend's message board

### Publish new code to OOD

After the previous code modification, our message board already has the function of viewing friends' message board information on its own message board. Before executing the publishing process, please open the cyfs.config.json file in the root directory and modify the version number of the version, such as upgrading from the original 1.0.1 to 1.0.2. After the modification is complete, republish the updated code to OOD.

After the release is completed, you and your friends can upgrade and install the message board app in the app store of the CYFS browser. Refer to the content of `Upgrade Dec App in CYFS browser and view` in the previous chapter.

### You can view your friend's message board

Open the CYFS browser, enter the front-end page of the message board, and add the value of peopleid to the page URL, such as: `cyfs://xxxx/meessage_baord/index.html?peopleid=xxxx`, you can see the content of your friend's message board! Is it very cool?
