# Introduction to the working principle of the simulator

The simulator simulates two zones in a single process, with two built-in oods and two devices for each ood, which can be used to develop and test some common use cases.

## mnemonic

- By default, zone-simulator derives all people and device information from mnemonics through the bip protocol, so the same set of mnemonics generates the same zone configuration information each time
- After the first startup, a random mnemonic is created by default and saved in {cyfs-root}/etc/zone-simulator/mnemonic; this configuration will be loaded after each startup in the future. If you need to create new zone desc information, delete this configuration file
- The execution speed of the debug version derived from the mnemonic key will be slower, and it usually takes one to two minutes to complete the execution.

## zone desc info

The zone information includes the following

- Two people desc information, representing two zones respectively
- Two ood desc messages, representing the ood of the two zones respectively
- Four device desc information, representing the two devices in each zone, and the ood of the zone where they are located to form an independent zone

After the zone-simulator is started, the above information will be written into the {cyfs-root}/etc/zone-simulator/desc_list directory, which corresponds to the mnemonic one-to-one. The desc information list generated by the same set of mnemonics is displayed every time the mnemonic is started. is the same, development and testing can directly rely on this information
For example, for mnemonic

```
million used drastic castle during top category wood street visa steak bachelor
```

The generated desc list is as follows

```
zone1 as follows:
people:5r4MYfFFQetBPDyeuBuDxPT7zowk9497SbpMQsZK2G19
ood:5aSixgLq9LVWbPByjRYgyjGicXtCN5S1nF1DMKZygLRE
device1:5aSixgPFg7R48VSo2b9PXcShofK3TUFUcHnHbtmB7fJr
device2: 5aSixgRaonSm1qWeCh8kNNrPGaB6pbETsqoN4Zb5Ev3H

zone2 as follows:
people:5r4MYfFGppKJXEjLvsgbKsibmTVLFZhL5cuZdEcXSr68
ood:5aSixgM12wHBfJBW1Q9n4xv1c5cu5g8hSxbrkhAQoojq
device1:5aSixgPdH5stj9pHrRVGXPNKiyrPsy8932BEr3zwFhXF
device2:5aSixgRx45jDU4Fg4ijYo3Yfo6RajtKSHgnGCVJUnk65
```

The above device_id and people_id information can be used directly as the target of interfaces such as router to interact with the corresponding protocol stack
For example, put_object to ood1 as the target:

```
const ood1 = DeviceId.from_base_58('5aSixgLq9LVWbPByjRYgyjGicXtCN5S1nF1DMKZygLRE').unwrap();
const target = ood1.object_id;
stack.put_object({
    ...,
    target
})
```

You can directly use the people_id of zone1 as the target:

```
const people1 = PeopleId.from_base_58('5r4MYfFFQetBPDyeuBuDxPT7zowk9497SbpMQsZK2G19').unwrap();
const target = people1.object_id;
stack.put_object({
    ...,
    target
})
```

## port information

Two zones, each zone contains one ood + two devices, so there are six protocol stacks in total. In order to facilitate external sdk calls, fixed ports are used, and the allocation is as follows

|    device     | bdt-port | http-port | ws-port |
| :-----------: | :------: | :-------: | :-----: |
|  zone1-ood1   |  20001   |   21000   |  21001  |
| zone1-device1 |  20002   |   21002   |  21003  |
| zone1-device2 |  20003   |   21004   |  21005  |
|  zone2-ood1   |  20010   |   21010   |  21011  |
| zone2-device1 |  20011   |   21012   |  21013  |
| zone2-device2 |  20012   |   21014   |  21015  |

Among them, http-port and ws-port can be used as the service interface of the local SharedObjectStack, directly use the open of SharedObjectStack, and pass in the corresponding interface.
For example, if you want to use the protocol stack of zone1-device2, you can use the following code to open:

```
const param = SharedObjectStackParam.new_with_ws_event_ports(21004, 21005).unwrap();
const stack = SharedObjectStack.open(param);
await stack.online();
```

## Scenario simulation

The above two zones can simulate most of the interaction scenarios in cyfs

- Interact with two devices in the zone, you can select device1 and device2 of the same zone
- Interact with the device and ood in the zone, you can choose any device+ood in the same zone
- Cross-zone interaction, select one device in each of zone1 and zone2
- To simulate a cross-zone server, use zone1 as the client and zone2 as the server, then use zone1-device1+ood2, and dec-service can be attached to ood2

# Debug with emulator

In the previous chapters, we used the real environment for testing. After each coding, we need to go through the release process to test, which will lead to low development and testing efficiency.
Now, we can use the emulator to greatly improve development efficiency.

## Start the emulator for the first time

### mac

In the tools/zone-simulator/mac folder in the project root directory, you can choose aarch64 or x86_64 to start the zone-simulator simulator program.

- The file may have no execute permission problem, just run it after executing `chmod 775 zone-simulator`.

### windows

Start the zone-simulator.exe simulator program in the tool/zone-simulator/windows folder in the project root directory.

## start debugging

### Run the front-end project

Enter the following command in the project root directory:

```shell
npm run dev
```

Visit http://localhost:8088 in the CYFS browser to see the front-end interface.

### Run Service

To run the Service, we need to use tsc to compile the ts files in the project into js. In the project root directory, open a new terminal and enter the following commands:

```shell
npx tsc
```

Next, copy the src/common/objs/obj_proto_pb.js file to deploy/src/common/objs/obj_proto_pb.js

Finally, enter the following commands in the root directory to open the two Zones simulated by the simulator:

```shell
npm run sim1
npm run sim2
```

---

It should be noted that these two commands are executed on two terminals respectively.

---

### Identity switching debugging

Open the src/www/initialize.ts file and use the first parameter of the `useSimulator` method to switch the identity. For example, to use the identity of `Zone1`, set it to `SimulatorZoneNo.FIRST`, and use the identity of `Zone1` to set is `SimulatorZoneNo.SECOND`.
Now, you can quickly debug in the emulator.

# More common CYFS debugging tips

It is recommended to print a log wherever possible errors can be made to quickly find the problem.

## Front-end debugging

1. View the print information in the CYFS browser console.

## Service side debugging

1. In the simulator environment, you can view the print information of the terminal and quickly find errors.

2. In the online environment, you can analyze the log files of the Service.

   - mac: ~/Library/cyfs/log/app/<app_name>
   - windows: C:\cyfs\log\app\<app_name>
   - linux: /cyfs/log/app/<app_name>

# Summary

Through the study of this chapter, you have mastered the knowledge and skills of debugging in the simulator environment, which will bring great convenience to your future development of Dec App.
